package com.google.gwt.query.client.css;

import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.dom.client.Style;
import static com.google.gwt.query.client.css.VerticalAlign.VAlignValue.create;

/**
 * This property affects the vertical positioning inside a line box of the boxes
 * generated by an inline-level element.
 */
public class VerticalAlign
    implements CssProperty<VerticalAlign.VAlignValue>, TakesPercentage,
    TakesLength {

  public static void init() {
    CSS.VERTICAL_ALIGN = new VerticalAlign();
    CSS.TOP = create("top");
    CSS.BOTTOM = create("bottom");
    CSS.BASELINE = create("baseline");
    CSS.SUB = create("sub");
    CSS.SUPER = create("super");
    CSS.TEXT_TOP = create("text-top");
    CSS.MIDDLE = create("middle");
    CSS.TEXT_BOTTOM = create("text-bottom");
  }

  public void set(Style s, VAlignValue value) {
    s.setProperty("verticalAlign", value.value());
  }

  public String get(Style s) {
    return s.getProperty("verticalAlign");
  }

  public void setPercentage(Style s, Percentage p) {
    s.setProperty("verticalAlign", p.value());
  }

  public void setLength(Style s, Length p) {
    s.setProperty("verticalAlign", p.value());
  }

  final static public class VAlignValue extends JavaScriptObject {

    protected VAlignValue() {
    }

    protected static VAlignValue create(String val) {
      return GWT.isScript() ? createWeb(val) : createHosted(val);
    }

    public String value() {
      return GWT.isScript() ? valueWeb() : valueHosted();
    }

    private static native VAlignValue createWeb(String val) /*-{
      return val;
    }-*/;

    private static native VAlignValue createHosted(String val) /*-{
      return [val];
    }-*/;

    private native String valueWeb() /*-{
       return this;
    }-*/;

    private native String valueHosted() /*-{
       return this[0];
    }-*/;
  }
}
