package com.google.gwt.query.client.css;

import com.google.gwt.dom.client.Style;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.GWT;

/**
 * This property specifies the content width of boxes generated by block-level
 * and replaced elements.
 *
 * This property does not apply to non-replaced inline-level elements. The
 * content width of a non-replaced inline element's boxes is that of the
 * rendered content within them (before any relative offset of children). Recall
 * that inline boxes flow into line boxes. The width of line boxes is given by
 * the their containing block, but may be shorted by the presence of floats.
 *
 * The width of a replaced element's box is intrinsic and may be scaled by the
 * user agent if the value of this property is different than 'auto'.
 */
public class Width implements TakesLength, TakesPercentage {

  public static void init() {
    CSS.WIDTH = new Width();
  }

  public void set(Style s, Length value) {
    s.setProperty("width", value.value());
  }

  public String get(Style s) {
    return s.getProperty("width");
  }

  public void setLength(Style s, Length p) {
    s.setProperty("width", p.value());
  }

  public void setPercentage(Style s, Percentage p) {
    s.setProperty("width", p.value());
  }
}